# Odhen Retail - Módulo Backend 

![](https://manage.teknisa.com/assets/images/logo.png)

**Figura 1** - Odhen Retail Logo

O repositório Odhen Retail - Módulo Backend é responsável por armazenar o código-fonte do back-end do módulo Odhen Retail - Módulo, também conhecido como MAN. Este é responsável por definir as rotas da aplicação: ([routes.json](http://code.zeedhi.com/teknisa/manbackend/tree/master/config/routes.json)), assim como seus respectivos Controllers (Onde as rotas chegam primeiramente), e serviços relacionados.

### Estrutura:

Este repositório utiliza o `composer` como gerenciador de pacotes e o `psr-0` para construir os namespaces da aplicação. O padrão MVC é utilizado em algumas rotas que fazem acesso direto ao banco de dados. Em outras, se utilizam dataSources (Arquivos JSON que definem um conjunto de dados) para se acessar e manipular entidades na base de dados.

### Namespaces:
    - Teknisa\MAN\
        - Controllers: "Recebem" e "Devolvem" as requisições feitas à aplicação.
        - Services: Camada de serviço da aplicação, responsável  
        - Factories: Repositórios que acessam o banco de dados. 
            - OMClient: Pacote de repositório que acessam a API do OM.
            - Retail: Pacote de repositórios exclusivos ao Retail.
        - Utilities: Pacote de Utilidades, responsável por diversas funcionalidades, como: fazer requisições, manipular objetos do Zeedhi, entre outros. 

### Classes notórias (Zeedhi e utilidades):

- Teknisa\MAN\Utilities\Util.php: Responsável por transformar qualquer tipo de  requisição do framework em um array associoativo PHP nativo.
- Zeedhi\Framework\Controller\CrudWithParameterBag: Ao se utilizar o datasource.json do Zeedhi, esta classe implementa as operações do CRUD com o banco de dados, sendo que o save vale tanto para o INSERT quanto o UPDATE.
- Zeedhi\Framework\DependencyInjection\InstanceManager: Classe responsável por instanciar e provocar a injeção de dependências na aplicação. Util para quando se precisa de um objeto construído via Symfony em um contexto estático (static).

### Instalar todo o projeto:
- Mais detalhes podem ser encontrados neste [link](http://code.zeedhi.com/teknisa/man).

-  No final esta deve ser a estrutura de pastas: 
    - management/ -> [Management](http://code.zeedhi.com/teknisa/management/)
    - management/modules/man/mobile ->  [MAN - Service](http://code.zeedhi.com/teknisa/man)
    - management/modules/man/ ->  [MAN - Service](http://code.zeedhi.com/teknisa/man)
    - management/modules/teknisa/man/ -> [MAN Backend - Este repositório.](http://code.zeedhi.com/teknisa/)
 


## Projetos Relacionados 

- [Odhen Retail Management][link_Produto]
- [Odhen Retail Management - Módulo Frontend][link_Frontend]
- [Odhen Retail Management - Módulo Backend][link_Backend]

## Instalação 
### Desenvolvimento:
Clone o microservice de produto desejado. A partir da raiz deste acesse a pasta **ci_scripts** do produto e execute o arquivo shell startMan. Caso também deseje, para realizar este procedimento de forma manueal, construa a seguinte estrutura de pastas com os respectivos reporsitórios 

- [Odhen Retail Management][link_Produto] -> ~/workfolder/management
- [Odhen Retail Management - Módulo Service][link_Service]  -> ~/workfolder/management/modules/man
- [Odhen Retail Management - Módulo Frontend][link_Frontend] -> ~/workfolder/management/modules/man/mobile
- [Odhen Retail Management - Módulo Backend][link_Backend] -> ~/workfolder/management/modules/teknisa/man

Então, rode os seguintes comandos para obter as dependências de cada microservice:
#
    cd backend;
    composer install;
    cd ../mobile;
    npm install;  
    bower install;
    grunt install;
    

### Produção
A configuração para deploy é feita utilizando as seguintes ferramentas:
- Grunt Build Environment (Para parâmetros iguais em todos clientes) -> grunt buildEnvironment
- Shell Script In Devops (Para parâmetros específicos de cada cliente)

### Referências Externas
- [NPM][link_NPM]
- [Bower][link_Bower]
- [Grunt][link_Grunt]
- [Composer][link_Composer]

[composer_file]: <http://code.zeedhi.com/teknisa/management/blob/master/backend/composer.json>
[bootstrap_file]: <http://code.zeedhi.com/teknisa/man/blob/master/backend/bootstrap.php>

[db_file]: <http://code.zeedhi.com/teknisa/man/blob/master/backend/config/db.xml.template>
[report_file]: <http://code.zeedhi.com/teknisa/management/blob/master/backend/config/report.xml.template>
[parameters_file]: <http://code.zeedhi.com/teknisa/man/blob/master/backend/config/parameters.xml.template>
[parameters_nfce_file]: <http://code.zeedhi.com/teknisa/man/blob/master/backend/config/parameters_nfce.xml>
[environment_file]: <http://code.zeedhi.com/teknisa/management/blob/master/backend/config/environment.xml>
[fileUploader_file]: <http://code.zeedhi.com/teknisa/man/blob/master/backend/config/fileUploader.xml.template> 
[acl_file]: <http://code.zeedhi.com/teknisa/man/blob/master/backend/config/acl.xml>

[link_Produto]: <http://code.zeedhi.com/teknisa/management>
[link_Frontend]: <http://code.zeedhi.com/teknisa/manfrontend>
[link_Backend]: <http://code.zeedhi.com/teknisa/manbackend>
[link_Service]: <http://code.zeedhi.com/teknisa/man>

[link_DevMedia]: <https://www.devmedia.com.br/introducao-a-nfc-e/33150>
[link_Composer]: <https://getcomposer.org/>
[link_NPM]: <https://www.npmjs.com/>
[link_Bower]: <https://bower.io/>
[link_Grunt]: <https://gruntjs.com/api/grunt>


