<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use \Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Notification;
use \Zeedhi\Framework\Util\Functions;
use \Exception as Exception;
use Zeedhi\Framework\DTO\Response\Message;

class cadImgProd {
    
    protected $CadImgProdService;
    protected $utilService;
    
    public function __construct (Services\CadImgProdService $CadImgProdService, Utilities\Util $utilService, Services\FileUploaderService $fileUploaderService) {
        $this->CadImgProdService = $CadImgProdService;
        $this->utilService = $utilService;
        $this->fileUploaderService = $fileUploaderService;
    }
    
    public function findProdImg(DTO\Request\Filter $request, DTO\Response $response){
        $result = [];
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $conditions['TCDPRODUTO']   = 'F';
        $conditions['CDPRODUTO']    = '';
        
        if(isset($conditions['CDFILIAL'])) {
            if ($conditions['TIPOPROD'] == 'T'){
                $conditions['TCDPRODUTO']   = 'T';
                $conditions['CDPRODUTOINI'] = '';
                $conditions['CDPRODUTOFIN'] = '';
            } else if ($conditions['TIPOPROD'] == 'L'){
                $conditions['CDPRODUTO']    = implode('_', $conditions['PRODUTOS']);
                $conditions['CDPRODUTOINI'] = '';
                $conditions['CDPRODUTOFIN'] = '';
            }
            
            $result = $this->CadImgProdService->findImgProducts($conditions);
        }
        
        return $response->addDataSet(new DataSet("select_prodFili", $result));
    }
    
    public function findProd(DTO\Request\Filter $request, DTO\Response $response){
        $result = [];
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $result = $this->CadImgProdService->findProducts($conditions);
        
        return $response->addDataSet(new DataSet("select_prodFili", $result));
    }
    
    public function saveImgProd(Row $params, DTO\Response $response){
        $rows = $params->getRow();
		try{
		    if (!isset($rows['LINKIMGPROD']) || $rows['LINKIMGPROD'] == '') {
    	        $rows['LINKIMGPROD'] = null;
    	    } else {
    	        if(isset($rows['LINKIMGPROD'][0]['name'])){
	                $rows['NMARQUIVO'] = $rows['LINKIMGPROD'][0]['name'];
        	    }
        	    else{
        	        if ($rows['TPMIDIA'] == 'Vazio'){
        	            $rows['NMARQUIVO'] = null;
        	        }
        	    }
        	    if(isset($rows['LINKIMGPROD'][0]['path'])){
                    if(Functions::arrayKeyExists('LINKIMGPROD', $rows)) {
                        $this->fileUploaderService->setImageForSave($rows, 'LINKIMGPROD', 'LINKIMGPROD');
                    }
                }
                else{
                    $this->fileUploaderService->setImageForSave($rows, 'LINKIMGPROD', 'LINKIMGPROD');
                }
    	    }
            $conditions = array(
    			':CDPRODUTO'	=> $rows['CDPRODUTO'],
    			':NRORG'		=> $rows['NRORG'],
    			':LINKIMGPROD'  => $rows['LINKIMGPROD'],
    			':NMIMAGEM'     => $rows['NMIMAGEM'],
    			':CDFILIAL'     => $rows['CDFILIAL'],
    			':CDIMAGEM'     => $rows['CDIMAGEM']
    		);
		    if ($this->CadImgProdService->validImagens($conditions)){
		        if(isset($rows['__is_new'])){
		            throw new Exception('Não foi possível realizar o cadastro, código de imagem ja cadastrado!');
		        } else {
		            $this->CadImgProdService->updateImgProd($conditions);
		        }
		    } else {
                $this->CadImgProdService->saveImg($conditions);
		    }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteImgProd(Row $params, DTO\Response $response){
        $rows = $params->getRow();
		try{
            $conditions = array(
    			':CDFILIAL'	    => $rows['CDFILIAL'],
    			':NRORG'		=> $rows['NRORG'],
    			':CDIMAGEM'     => $rows['CDIMAGEM'],
    			':CDPRODUTO'	=> $rows['CDPRODUTO']
    		);
            $this->CadImgProdService->deleteImgProd($conditions);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function findImagens(DTO\Request\Filter $request, DTO\Response $response){
        $result = [];
        $conditions = $this->utilService->getParamsAsArray($request);
        $result = $this->CadImgProdService->findImagens($conditions);
        
        return $response->addDataSet(new DataSet("select_prodFili", $result));
    }
    
}
