<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Utilities\Jobs;

class ConsolidacaoAutomatica extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceManager;
    protected $crudService;
    protected $consolidacaoAutService;
    protected $salesMessage;
    
    public function __construct($dataSourceManager, $crudService, Services\ConsolidacaoAutomatica $consolidacaoAutService, Utilities\SalesMessage $salesMessage) {
        $this->dataSourceManager      = $dataSourceManager;
        $this->crudService            = $crudService;
        $this->consolidacaoAutService = $consolidacaoAutService;
        $this->salesMessage           = $salesMessage;
    }
    
    public function buscaConsolidacoes(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $criterioestabelec = $this->consolidacaoAutService->buscaConsolidacoes($params);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $criterioestabelec));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function saveConsolidacoes(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            
            $data            = $row[0]['HRCONSVENAUT'];
            $frequencia      = $row[0]['IDFREQCONSVEN'];
            $cdFilial        = $row[0]['CDFILIAL'];
            $cdLoja          = $row[0]['CDLOJA'];
            $dtIniConsVenAut = $row[0]['DTINICONSVENAUT'];
            $cdFiliProc      = $row[0]['CDFILIPROC'];
            $cdOperador      = $row[0]['CDOPERADOR'];
            $nrorg           = $row[0]['NRORG'];
            $nomeEmpresa     = $this->consolidacaoAutService->buscaNomeEmpresa($nrorg);
            $nomeEmpresa     = str_replace(" ", "_", $nomeEmpresa);
            $nomeJob         = 'CONSOLIDA_'.$nomeEmpresa.'_'.$nrorg.'_'.$cdFilial.'_'.$cdLoja;
            $descricao       = 'Consolidação Automática da Empresa: '.$nomeEmpresa.', Unidade: '.$cdFilial.', Loja: '.$cdLoja;
            
            $dataHora = $this->alteraHorario($data, $frequencia);
            
            $cdTipoReti = $this->consolidacaoAutService->buscaTipoRetiradaFilial($cdFilial, $nrorg);
            
            if(empty($cdTipoReti)) {
                throw new \Exception("A Unidade $cdFilial não possui 'Tipo de Retirada'. É necessário cadastrar um Tipo de Retirada na tela Parâmetros da Unidade > Unidade > Consolidação de vendas > Padrões da Consolidação > Tipo de retirada padrão Utilizada na consolidação de Vendas.");
            } else {
                $params[0]['value'] = $cdFilial;
                $params[1]['value'] = $cdLoja;
                $params[2]['value'] = $nrorg;
                
                $result = $this->consolidacaoAutService->buscaConsolidacoes($params);
                
                if(count($result) > 0) {
                    $row[0]["__is_new"] = false;
                } else {
                    $row[0]["__is_new"] = true;
                }
                
                $arrayConsolidacaoAutomatica = [
                    'NOMEJOB'       => $nomeJob, 
                    'DESCRICAO'     => $descricao, 
                    'DATAHORA'      => $dataHora, 
                    'CDFILIAL'      => $cdFilial, 
                    'CDLOJA'        => $cdLoja, 
                    'DATAINICIAL'   => $dtIniConsVenAut, 
                    'CDFILIPROC'    => $cdFiliProc, 
                    'CDOPERADOR'    => $cdOperador, 
                    'NRORG'         => $nrorg, 
                    'CDTIPORETI'    => $cdTipoReti[0]['CDTIPORETI'],
                    'URL_JENKINS'   => $row[0]['URL_JENKINS'],
                    'USER_JENKINS'  => $row[0]['USER_JENKINS'],
                    'TOKEN_JENKINS' => $row[0]['TOKEN_JENKINS']
                ];
                
                if($row[0]["__is_new"] == true) {
                    $this->consolidacaoAutService->insertConsolidacoes($row);
                    $message = array($this->salesMessage->getMessage('criado'));
                    
                    // cria o job
                    $this->criaJob($arrayConsolidacaoAutomatica);
                    
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $message));
                } else {
                    $criterioestabelec = $this->consolidacaoAutService->updateConsolidacoes($row);
                    $message = array($this->salesMessage->getMessage('alterado'));
                    
                    // remove o job
                    // $this->removeJob($nomeJob, $arrayConsolidacaoAutomatica['URL_JENKINS']);
                    
                    // cria o job
                    $this->criaJob($arrayConsolidacaoAutomatica);
                    
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $message));
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Metodo alteraHorario() - para alterar o horário para um formato 
     *                          que o Jenkins aceita.
     * @param $horario      - Horário definido no frontend
     * @param $intervalo    - Intervalo definido no frontend
     * @return $novoHorario - Horário no formato do Jenkins
     */
    private function alteraHorario($horario, $intervalo) {
        $novoHorario = "* * * * *";
        $horario     = explode(':', $horario);
        $hora        = $horario[0];
        $minuto      = $horario[1];
        
        if($intervalo === "D") {
            $novoHorario = $minuto . " " . $hora . " * * *";
        } else if($intervalo === "I") {
            $novoHorario = $minuto . " H/" . $hora . " * * *";
        }
        
        return $novoHorario;
    }
    
    /**
     * Metodo criaJob() - Cria o Job no Jenkins passando os parâmetros para 
     *                    o arquivo que realizará a consolidação das vendas periodicamente.
     * 
     * @param $params - Array com todos os parâmetros necessários
     */
    private function criaJob($params) {
        try {
            $dataInicialApuracao = explode(' ', $params['DATAINICIAL']);
            
            /* 
                Exemplos:
                $urlCliente = "https://retail.teknisa.com"; // SAAS
                $urlCliente = "https://giraffas-retail.teknisa.com"; // HOSTING
            */
            $urlCliente = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]";
            
            $path = getcwd();
            $rota = "/index.php/ExecConsVen";
            
            if(strpos($path, "/workfolder/management/modules/man/backend/service")) {
            	// dev
            	$urlDevPath = explode("home/developer", $path);
            	
            	$rotaPublica = $urlDevPath[1] . $rota;
            } else if(strpos($path, "/odhen/management/modules/man/backend/service")) {
            	// test
            	$rotaPublica = "/odhen/management/modules/man/backend/service" . $rota;
            } else {
            	// prod
            	$rotaPublica = "/backend_man" . $rota;
            }
            
            $metodo = "curl --location --request POST '". $urlCliente . $rotaPublica . "' --header 'CDFILIAL: ". $params['CDFILIAL'] ."' --header 'CDLOJA: ". $params['CDLOJA'] ."' --header 'DATA: ". $dataInicialApuracao[0] ."' --header 'CDFILIPROC: ". $params['CDFILIPROC'] ."' --header 'CDOPERADOR: ". $params['CDOPERADOR'] ."' --header 'NRORG: ". $params['NRORG'] ."' --header 'CDTIPORETI: ". $params['CDTIPORETI'] ."'";
            
            $jenkins = new Jobs($params['NOMEJOB']);
			
            $jenkins->setType(Jobs::TYPE_JOB_MULT_CONF);
            $jenkins->setUrlJenkins($params['URL_JENKINS'] . "/");
            $jenkins->setUrlConfig(null);
            $jenkins->setUrlCreateItem(null);
            $jenkins->setPeriod($params['DATAHORA']);
            $jenkins->setDescription($params['DESCRICAO']);
            $jenkins->setMethod($metodo);
            
            $jenkins->createJob($params['USER_JENKINS'], $params['TOKEN_JENKINS']);
            $jenkins->configuration($params['USER_JENKINS'], $params['TOKEN_JENKINS']);
            $jenkins->enable($params['USER_JENKINS'], $params['TOKEN_JENKINS']);
        } catch(\Exception $e) {
            echo $e->getMessage();
            die;
        }
    }
    
    /**
     * Metodo removeJob() - Remove o job do Jenkins.
     * 
     * @param $nomeJob    - nome do job.
     * @param $urlJenkins - Url do jenkins
     */
    private function removeJob($nomeJob, $urlJenkins) {
        $jenkins = new Jobs($nomeJob);
        
        $jenkins->setUrlJenkins($urlJenkins . "/");
        $jenkins->setUrlDelete(null);
        $jenkins->deleteJob();
    }
    
    public function removeConsolidacoes(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $this->consolidacaoAutService->removeConsolidacoes($row);
            
            $message = array($this->salesMessage->getMessage('excluido'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

}